/*
 * This file is part of Bipscript-IDE.
 *
 * Bipscript-IDE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript-IDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript-IDE.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "audioengine.h"

void jackInfoShutdownCallback(void *ae) { static_cast<AudioEngine *>(ae)->onShutdown(); }

AudioEngine::AudioEngine() {
    thread.start();
    QObject::connect(&thread, SIGNAL(finished()), &thread, SLOT(deleteLater()));
}

AudioConfiguration AudioEngine::connect() {
    jack_options_t options = JackNullOption;
    jack_status_t status;
    client = jack_client_open("Bipscript IDE", options, &status, nullptr);
    jack_on_shutdown(client, &jackInfoShutdownCallback, this);
    AudioConfiguration ret;
    ret.framesPerSecond = jack_get_sample_rate(client);
    return ret;
}

void AudioEngine::disconnect() { jack_client_close(client); }

void AudioEngine::transportStart() { jack_transport_start(client); }

void AudioEngine::transportStop() { jack_transport_stop(client); }

void AudioEngine::reposition(uint frame) { jack_transport_locate(client, frame); }

AudioTimerService *AudioEngine::timerService() {
    timer = new AudioTimerService(client);
    timer->moveToThread(&thread);
    timer->start();
    return timer;
}
