/*
 * This file is part of Bipscript-IDE.
 *
 * Bipscript-IDE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript-IDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript-IDE.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef AUDIOENGINE_H
#define AUDIOENGINE_H

#include "audiotimerservice.h"

#include <QThread>
#include <jack/jack.h>

struct AudioConfiguration {
    unsigned int framesPerSecond;
};

class AudioEngine {
    jack_client_t *client;
    QThread thread;
    AudioTimerService *timer;
    AudioEngine();

  public:
    static AudioEngine &instance() {
        static AudioEngine *instance = nullptr;
        if (instance == nullptr) {
            instance = new AudioEngine();
        }
        return *instance;
    }
    AudioConfiguration connect();
    void disconnect();
    void transportStart();
    void transportStop();
    void reposition(uint frame);
    void onShutdown() {
        if (timer) {
            timer->stop();
        }
    }
    AudioTimerService *timerService();
};

#endif // AUDIOENGINE_H
