/*
 * This file is part of Bipscript-IDE.
 *
 * Bipscript-IDE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript-IDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript-IDE.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "audiotimerservice.h"

#include <QDebug>
#include <unistd.h>

AudioTimerService::AudioTimerService(jack_client_t *c)
    : cancelled(false), mrolling(false), client(c) {
    QObject::connect(this, SIGNAL(startTrigger()), this, SLOT(process()));
    QObject::connect(this, SIGNAL(finished()), this, SLOT(deleteLater()));
}

void AudioTimerService::process() {
    jack_position_t jack_pos;
    jack_transport_state_t transport_state;

    while (!cancelled) {
        // query transport
        transport_state = jack_transport_query(client, &jack_pos);

        // output position
        Position position;
        position.frame = jack_pos.frame;
        position.msec = jack_pos.frame * static_cast<uint64_t>(1000) / jack_pos.frame_rate;
        if (jack_pos.valid & JackPositionBBT) {
            position.bar = jack_pos.bar;
            position.beat = jack_pos.beat;
            position.tick = jack_pos.tick;
        } else {
            position.bar = 0;
            position.beat = 0;
            position.tick = 0;
        }
        emit transport(position);

        // output state change
        bool state(transport_state == JackTransportRolling);
        if (state != mrolling) {
            mrolling = state;
            emit rolling(mrolling);
        }
        usleep(200000);
    }
    emit finished();
}
