#ifndef OBJECTGRAPH_H
#define OBJECTGRAPH_H

#include "graphnode.h"
#include "oscmessage.h"

#include <QGraphicsView>
#include <QTimer>
#include <map>

class ObjectGraph : public QGraphicsView {
    Q_OBJECT

    QGraphicsScene *scene;
    std::map<int64_t, GraphNode *> nodeMap;
    std::map<int64_t, Connector *> connectorMap;
    std::map<int64_t, Connection *> connectionMap;
    std::map<int64_t, Clock *> clockMap;
    QTimer *timer;
    GraphItem *mselected;

  public:
    ObjectGraph(QWidget *parent = 0);
    void cconnect(int64_t port1, int64_t port2);
    void addClock(int64_t node, QString &name, float tempo, uint16_t signum, uint16_t sigden);
    void addLv2Plugin(int64_t mixer, QString &name, int inputs, int outputs, int midiins,
                      int midiouts);
    void addProcessorMidi(int64_t hash, int midiins, int midiouts);
    void addMidiEvent(int64_t hash, int64_t clock, int64_t eid, float measure, QString &name,
                      int arg1, int arg2);
    void addScriptEvent(int64_t clock, int64_t eid, float measure, QString &name);
    void removeMidiEvent(int64_t hash, int64_t clock, int64_t eid);
    void clearMidiEvents(int64_t port, int64_t clock);
    void removeScriptEvent(int64_t clock, int64_t eid);
    void clearScriptEvents(int64_t node);
    void addMixer(int64_t hash, QString &name, int inputs, int outputs);
    void addInput(int64_t hash, QString &name, int channels);
    void addOutput(int64_t hash, QString &name, int channels);
    void addMidiInput(int64_t hash, QString &name, int channels);
    void addMidiOutput(int64_t hash, QString &name, int channels);
    void mixerGain(int64_t mixer, int input, int output, float gain);
    void nodeInputs(int64_t node, int64_t *ports);
    void nodeOutputs(int64_t node, int64_t *ports);
    void nodeMidiInputs(int64_t nodeid, int64_t *ports);
    void nodeMidiOutputs(int64_t nodeid, int64_t *ports);
    // recurring
    void levels(int64_t node, float level[]);
    void midilevels(int64_t nodeid, uint16_t index, float level);
    void mixerGains(int64_t mixer, int input, float *gains);
    void clockTime(int64_t node, float measure);
    void clockUpdate(int64_t node, bool running, float measure, float tempo, uint16_t signum,
                     uint16_t sigden);
    // events
    void onSelection();
    // cleanup
    void finishClock(int64_t hash);
    void removeNode(int64_t hash);
    void scriptStart();
    void completed();
    void clockTick();
  signals:
    void selected(GraphItem *);

  private:
    void addNode(GraphNode *node1, int64_t hash);
    void layout();
    double timenow();
};

#endif // OBJECTGRAPH_H
