/*
 * This file is part of Bipscript-IDE.
 *
 * Bipscript-IDE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript-IDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript-IDE.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef OUTPUTWIDGET_H
#define OUTPUTWIDGET_H

#include "objectgraph/objectgraph.h"
#include "oscserver.h"

#include <QTextCharFormat>
#include <QUrl>
#include <QWidget>

namespace Ui {
class OutputWidget;
}

class UtfChar { // helper for broken utf-8 chars
    static const uint MAX_SIZE = 7;
    char buffer[MAX_SIZE];
    uint index = 0;
    uint msize = 0;

  public:
    void append(char);
    bool complete();
    char *bytes();
};

class OutputWidget : public QWidget {
    Q_OBJECT

    int mpid;
    QString lineBuffer;
    QTextCharFormat errorFormat; // stderr
    QTextCharFormat outFormat;   // stdout
    QTextCharFormat inFormat;    // stdin
    UtfChar mutfchar;

  public:
    explicit OutputWidget(QWidget *parent = nullptr);
    ~OutputWidget();
    int pid() { return mpid; }
    int port() { return osc->port(); }

  protected:
    void keyPressEvent(QKeyEvent *event) override;

  public slots:
    void running(int);
    void write(const char *);
    void writeError(QString);
    void endScript(bool);
    void killScript(bool);
    void completed(int);
    void message(OscMessage);
    void restartScript(bool);
    void selected(GraphItem *);

  private slots:
    void stdoutLink(const QUrl &url) { emit linkClicked(url); }
    void on_connectorCombo_currentIndexChanged(int index);
    void on_clockCombo_currentIndexChanged(int index);

  signals:
    void linkClicked(const QUrl &);
    void codeError(const QString &path, int line);
    void textInput(QString);

  private:
    Ui::OutputWidget *ui;
    OscServer *osc;
    ObjectGraph *mgraph;
    GraphItem *selectedItem;
    void parseLine(int start);
    void updateModel(int connectoridx, int clockidx);
};

#endif // OUTPUTWIDGET_H
