/*
 * This file is part of Bipscript-IDE.
 *
 * Bipscript-IDE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript-IDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript-IDE.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef PROJECTCONFIG_H
#define PROJECTCONFIG_H

#include "namedfile.h"
#include "projectlist.h"

#include <QDir>
#include <QList>
#include <QUuid>

#define BIP_CONFIG_DIR ".bip"
#define BIP_IDE_CONFIG_DIR BIP_CONFIG_DIR "/ide"

class RunConfig {
    QFileInfo mpath;
    QString mname;
    QString margs;

  public:
    RunConfig() {}
    RunConfig(const QString &name, const QString &path, const QString &args)
        : mpath(path), mname(name), margs(args) {}
    RunConfig(const QString &path, const QString &args) : mpath(path), margs(args) {
        mname = mpath.fileName();
    }
    QString args() const { return margs; }
    void args(const QString &args) { margs = args; }
    QString name() const { return mname; }
    void name(const QString &name) { mname = name; }
    QFileInfo path() const { return mpath; }
    QString fullpath() const { return path().canonicalFilePath(); }
    bool operator==(const RunConfig &other) { return other.mpath == mpath && other.margs == margs; }
};
Q_DECLARE_METATYPE(RunConfig)

class Project {
    QString mid; // TODO: const
    QString mname;
    QList<QString> mfolders;
    QList<QString> mopenFiles;
    QList<RunConfig> mrunconfigs;
    int mactiveRun;
    QString mexecutable;
    QString mactiveFile;

  public:
    Project(QString id) : mid(id), mactiveRun(0) { this->executable("/usr/bin/bipscript"); }
    bool exists();
    QString name() { return mname; }
    void name(QString name) { mname = name; }
    QString id() const { return mid; }
    QList<QString> &folders() { return mfolders; }
    QList<QString> &openFiles() { return mopenFiles; }
    QList<RunConfig> &runConfigs() { return mrunconfigs; }
    QString &executable() { return mexecutable; }
    void executable(const QString &exec) { mexecutable = exec; }
    QString &activeFile() { return mactiveFile; }
    void activeFile(const QString &active) { mactiveFile = active; }
    int activeRun() { return mactiveRun; }
    void activeRun(int activeRun) { mactiveRun = activeRun; }
};

struct ProjectId {
    QString id;
    QString name;
    friend bool operator<(const ProjectId &l, const ProjectId &r) { return l.name < r.name; }
};

class ProjectConfig {

  public:
    bool save(Project &project);
    bool remove(QString id);
    Project *load(QString id);
    QList<ProjectId> names();
    int count();
    QString filename(QString id);
    Project *findByLocation(QString folder);
    QString createId();

  private:
    QDir getConfigDir();
};

#endif // PROJECTCONFIG_H
