/*
 * This file is part of Bipscript-IDE.
 *
 * Bipscript-IDE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript-IDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript-IDE.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef SEARCHTREEITEM_H
#define SEARCHTREEITEM_H

#include <QTreeWidgetItem>

#include "editorlink.h"

class SearchTreeFileItem : public QTreeWidgetItem {
    int mresults;
    QString mpath;
    QString mfilename;

  public:
    SearchTreeFileItem(const QString &path, const QString &filename, int results)
        : QTreeWidgetItem(static_cast<QTreeWidgetItem *>(nullptr),
                          QStringList(header(filename, results)), QTreeWidgetItem::UserType),
          mresults(results), mpath(path), mfilename(filename) {
        setToolTip(0, path);
    }
    int results() { return mresults; }
    void results(int results) {
        mresults = results;
        setText(0, header(mfilename, mresults));
    }
    QString &path() { return mpath; }

  private:
    static QString header(const QString &filename, int results) {
        return QString("%1 (%2)").arg(filename).arg(results);
    }
};

class SearchTreeItem : public QTreeWidgetItem {
    EditorLocation mloc;

  public:
    SearchTreeItem(QTreeWidgetItem *parent, EditorLink &link)
        : QTreeWidgetItem(parent, QStringList(link.text()), QTreeWidgetItem::UserType),
          mloc(link.location()) {}
    EditorLocation &location() { return mloc; }
};
#endif // SEARCHTREEITEM_H
