/*
 * This file is part of Bipscript-IDE.
 *
 * Bipscript-IDE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript-IDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript-IDE.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef SEARCHWIDGET_H
#define SEARCHWIDGET_H

#include "searchterm.h"
#include "searchtreeitem.h"
#include <QWidget>

namespace Ui {
class SearchWidget;
}

class SearchWidget : public QWidget {
    Q_OBJECT

  public:
    explicit SearchWidget(QWidget *parent = nullptr);
    void newFileSearch(QString term) { newSearch(term, false); }
    void newProjectSearch(QString term) { newSearch(term, true); }
    void addResults(const QString &path, const QString &filename, QList<EditorLink> results,
                    bool open);
    void updateResults(const QString &path, QList<EditorLink> results);
    void clear();
    ~SearchWidget();

  signals:
    void searchCurrent(SearchTerm);
    void searchNext(SearchTerm);
    void searchProject(SearchTerm);
    void replaceNext(SearchTerm, QString);
    void navigateResult(EditorLocation &);
    void clearSearch();

  private slots:
    void resultTreeClicked(QTreeWidgetItem *, int);
    void on_searchButton_clicked();
    void on_searchLineEdit_returnPressed();
    void on_searchLineEdit_textEdited(const QString &arg1);
    void on_replaceButton_clicked();
    void on_refreshButton_clicked();
    void on_fileScopeButton_clicked();
    void on_projectScopeButton_clicked();
    void on_searchSensistiveCheck_stateChanged(int arg1);

  private:
    Ui::SearchWidget *ui;
    bool mnewSearch;
    int count;
    int fcount;
    void doSearch();
    void newSearch(QString, bool);
    void showCount();
};

#endif // SEARCHWIDGET_H
